/******************************************************************
Module : CfgMgr.cpp    - preparation of config data to be written
                         to a binary file
             Written 1998,1999 by Dmitri Leman
Purpose: COM/OLE tracer and NT kernel mode tracer.
******************************************************************/

#include "Headers.h"
#include "Trace.h"

#include "CfgMgr.h"

/*-----------------------------------------------------------------

   FUNCTION: ASM_APIFunctionStub::vInitForFile

   PURPOSE:  Initialize function stub when it is first prepared
    for our binary file. Final preparation of the stub will be made
    by function ASM_APIFunctionStub::vAdjustOnDllLoad.
      
   PARAMETERS:                   .
                    The following offsets are from the beginning of
                    configuration data in the binary file
                                      
   ULONG   p_dwModuleNameOffset   Place in binary file where 
                                  module name is stored
   ULONG   p_dwFunctionNameOffset Where the function name is stored
   int     p_iFunctionIndex       Index of the function in VTBL
                                  (for interfaces only)
   ULONG   p_dwParamsOffset       Place where encoded function 
                                  parameters are stored
_________________________________________________________________*/
void ASM_APIFunctionStub::vInitForFile
(
    ULONG   p_dwModuleNameOffset,
    ULONG   p_dwFunctionNameOffset,
    int     p_iFunctionIndex,//non-negative for interfaces
    ULONG   p_dwParamsOffset
)
{
    m_dwSignature = STUB_SIGNATURE;
    m_dwModuleNameOffset = p_dwModuleNameOffset;
    m_dwParamsOffset = p_dwParamsOffset;
    m_dwFunctionNameOffset = p_dwFunctionNameOffset;
    m_iFunctionIndex = p_iFunctionIndex;
    
    m_Code.instr_PUSH = PUSH_INSTR;
    m_Code.value_orig_address = 0;

    m_Code.instr_CALL = CALL_INSTR;
    m_Code.offset_LogFunction = 0;

    m_Code.instr_RET = RET_INSTR;
}//void ASM_APIFunctionStub::vInitForFile
