/******************************************************************
Module : CfgMgr.h Declaration of data structures which will 
                  hold our configuration data in the binary file
             Written 1998,1999 by Dmitri Leman
Purpose: COM/OLE tracer and NT kernel mode tracer.
*****************************************************************/

#ifndef _CFGMGR_H_
#define _CFGMGR_H_

//The following classes and structures has no virtual tables.
//They will be stored directly in the binary configuration file.
//Initially all pointers stored as offsets relative to the
//location of the structure itself.These offsets will be replaced 
//by pointers during DLL(or driver) initialization.

#define PUSHAD_INSTR 0x60
#define PUSH_INSTR   0x68
#define CALL_INSTR   0xE8
#define RET_INSTR    0xC3

#define STUB_SIGNATURE 0x62757473 //"stub"

#pragma pack(push,1)

struct ASM_APIFunctionStubCode //12 bytes
{
    UCHAR   instr_PUSH;          //Push original address
    ULONG   value_orig_address;  //the original address itself

    UCHAR   instr_CALL;        //Call the IntrerceptEntry function
    ULONG   offset_LogFunction;//which is on this address

    UCHAR   instr_RET;    //transfer control to the original address

    UCHAR   padding;
};//struct ASM_APIFunctionStubCode


struct ASM_APIFunctionStub //32 bytes
{
    ULONG   m_dwSignature; //STUB_SIGNATURE
    ULONG   m_dwFunctionNameOffset;//offset of function name
    int     m_iFunctionIndex;    //non-negative for interfaces
    ULONG   m_dwModuleNameOffset;  //offset of module name
    ULONG   m_dwParamsOffset;//array of encoded parameters

    ASM_APIFunctionStubCode m_Code;

    void vInitForFile
    (
        ULONG   p_dwModuleNameOffset,
        ULONG   p_dwFunctionNameOffset,
        int     p_iFunctionIndex,//non-negative for interfaces
        ULONG   p_dwParamsOffset
    );
    void vAdjustOnDllLoad(UCHAR*  p_pMem, void *  p_pLogFunction);
    void vShortcut();
    ULONG dwGetOriginalFunctionAddress()
    {
        return m_Code.value_orig_address;
    }
    void vSetOriginalFunctionAddress(ULONG p_dwAddress)
    {
        m_Code.value_orig_address = p_dwAddress;
    }

};//struct ASM_APIFunctionStub

struct MyInterface
{
public:
    void vDllInit(UCHAR*  p_pMem);

    LPCSTR pszGetName() {return (LPCSTR)m_dwOffsetName;}
    ASM_APIFunctionStub * pGetStub(int p_iIndex);

public:
    int     m_iNumFunctions;
    int     m_iMaxFunctionIndex;//largest index of functions in the
                                //VTable for this interface.
    GUID    m_ID;

    ULONG   m_dwOffsetFunctions;//offset of array of pointers 
        //to ASM_APIFunctionStub. This array is a part of one 
        //large array where all stubs are stored. It is pointed by 
        //ConfigManager::m_dwOffsetStubs

    ULONG   m_dwOffsetName;//offset to interface name relative to 
     //this structure (or pointer after DLL(driver) initialization)

    ULONG   m_dwModuleBase;        //current module base

    ULONG   m_dwModuleBasePrevious;//temporary storage for module base
                                   //when we rescan the module list
};//struct MyInterface

class FunctionUnderConstruction;
class ConfigManager
{
public:

protected:
    int         m_iTotalNumInts;//interfaces and Modules
    int         m_iNumInterfaces;//interfaces are not used for the
                            //kernel spy and this value == 0
    int         m_iNumModules;
    int         m_iIUnknownIndex;
    int         m_iIDispatchIndex;

    int         m_iNumStubs;   //total number of stubs
    int         m_iNumInterfStubs;//number of stubs for interfaces
    int         m_iNumExpStubs;//number of export function stubs.
    ULONG       m_dwOffsetStubs;//Offset to an array of pointers
            //to all ASM_APIFunctionStub created for API functions 
            //and interfaces. Interface stubs are located first,
            //exported - second

    //The next 2 members are used only during preparation of data
    int         m_iInterfacesFilled;
    int         m_iStubsFilled;

    MyInterface m_ArrayInterfaces[1];//size m_iTotalNumInts
        //Interfaces goes first, Modules - second

    friend class FunctionUnderConstruction;
    friend class InterfaceUnderConstruction;
    friend class AllInterfacesUnderConstruction;
};//class ConfigManager

#pragma pack (pop)

#endif //_CFGMGR_H_
