/******************************************************************
Module : InterMap.h - collections of modules, interfaces, functions 
                     and parameters used during config file parsing
             Written 1998,1999 by Dmitri Leman
Purpose: COM/OLE tracer and NT kernel mode tracer.
******************************************************************/

#ifndef _INTERMAP_H_
#define _INTERMAP_H_

#pragma warning(disable:4786)
#include <string>
#include <vector>
#include <map>
using namespace std;

class ConfigManager;

template<class T>
class NamedArray
{
public:
    virtual ~NamedArray(){vFree();}

    int iGetSize();
    int iFind(const string & p_rstrName, 
                                T ** p_ppReturnObject = NULL);
    T * pGet(int p_iIndex);
    BOOL bAdopt(const string & p_rstrName, 
        T * p_pObject,    //must be created by new operator.
                                //If the function returns TRUE, 
                                //the array takes ownership of this
                                //object and may delete it.
        int  * p_piReturnIndex);
        
    void vFree();
protected:
    typedef vector<T*> Array;
    typedef map<string, int> MapNameToIndex;

    Array           m_Array;
    MapNameToIndex  m_Map;
};//class NamedArray

enum    ParamType;
class ParamsUnderConstr
{
public:
    ParamsUnderConstr() : m_iNumParameters(0),m_byCommonFlags(0){}

    BOOL bAddParam(ParamType p_Type, BYTE p_byFlags, 
        BOOL p_bExtraBytePresent, BYTE  p_byExtra);
    int iExport(PBYTE   p_pMem, int p_iTotalSize,int p_iCurOffset);
protected:
    BYTE            m_byCommonFlags;
    int             m_iNumParameters;
    vector<BYTE>    m_EncodedParams;
};//class ParamsUnderConstr

class FunctionUnderConstruction
{
public:
    FunctionUnderConstruction
        (const string & p_rstrName, int p_iIndex)
        :m_strName(p_rstrName), m_iIndex(p_iIndex){}
    
    ParamsUnderConstr & Params() {return m_Params;}
    int iExport(PBYTE   p_pMem, int p_iTotalSize, int p_iCurOffset,
        ConfigManager * p_pCfg, DWORD   p_dwModuleNameOffset);
protected:
    string              m_strName;
    int                 m_iIndex;
    ParamsUnderConstr   m_Params;
};//class FunctionUnderConstruction

class InterfaceUnderConstruction
: public NamedArray<FunctionUnderConstruction>
{
public:
    enum TypeInterface
    {
        TI_Interface,
        TI_ForwardReference,
        TI_DLL
    };
    InterfaceUnderConstruction
        (TypeInterface p_Type,
        const string & p_rstrName, const IID  & m_rID)
        :m_Type(p_Type), m_ID(m_rID), 
        m_strName(p_rstrName), m_iMaxFunctionIndex(-1), 
        m_iBaseIndex(-1), m_iNumFunctions(0) {}
    InterfaceUnderConstruction
        (TypeInterface p_Type, const string & p_rstrName)
        :m_Type(p_Type), 
        m_strName(p_rstrName), m_iMaxFunctionIndex(-1),
        m_iBaseIndex(-1), m_iNumFunctions(0) 
    {memset(&m_ID,0,sizeof(m_ID));}

    BOOL bReplaceForwardRefByInterface(
        const InterfaceUnderConstruction * l_pInt);

    BOOL bAddFunction
    (
        const string &  p_rstrFunctionName,
        int             p_iFunctionIndex,
        FunctionUnderConstruction ** p_ppReturnFunction
    );
    int iExport(NamedArray<InterfaceUnderConstruction> &p_rAllIntr,
        PBYTE   p_pMem, int p_iTotalSize, int p_iCurOffset,
        ConfigManager * p_pCfg);
    int iExportFunctions(
        NamedArray<InterfaceUnderConstruction> & p_rAllIntr,
        PBYTE   p_pMem, int p_iTotalSize, int p_iCurOffset,
        ConfigManager * p_pCfg, DWORD   p_dwModuleNameOffset);

    void vAcceptBase(int p_iBaseIndex, 
        const InterfaceUnderConstruction & p_rBase)
    {
        m_iBaseIndex = p_iBaseIndex;
        m_iMaxFunctionIndex = p_rBase.m_iMaxFunctionIndex;
        m_iNumFunctions = p_rBase.m_iNumFunctions;
    }
    int iGetNumFunctions() {return m_iNumFunctions;}
protected:
    TypeInterface m_Type;
    IID     m_ID;
    int     m_iMaxFunctionIndex;
    int     m_iBaseIndex;
    int     m_iNumFunctions;//including base
    string  m_strName;
};//class InterfaceUnderConstruction

class AllInterfacesUnderConstruction
//: public NamedArray<InterfaceUnderConstruction>
{
public:
    AllInterfacesUnderConstruction();

    int iAddInterface(const string & p_rstrName, 
        const string & p_rstrBase, const IID & p_rID,
        InterfaceUnderConstruction ** p_ppReturnInterface);
    int iAddOrFindDLL(const string & p_rstrName,
        InterfaceUnderConstruction ** p_ppReturnInterface);
    int iGetInterfaceReference(const string & p_rstrName);
    int iAddIUnknown();
    int iExportInterfaces (
        NamedArray<InterfaceUnderConstruction> & p_rInts,
        PBYTE   p_pMem, int     p_iTotalSize, int     p_iCurOffset,
        ConfigManager * p_pCfg);
    int iExport(PBYTE   p_pMem, int p_iTotalSize,int p_iCurOffset);
protected:
    NamedArray<InterfaceUnderConstruction> m_Interfaces;
    NamedArray<InterfaceUnderConstruction> m_DLLs;

    int m_iIUnknownIndex;
    int m_iIDispatchIndex;
};//AllInterfacesUnderConstruction
#endif//_INTERMAP_H_
