/******************************************************************
Module : Log.h - class Logger which has several static function
               for processing intercepted function call and returns
             Written 1998,1999 by Dmitri Leman
Purpose: Purpose: NT kernel mode tracer.
******************************************************************/
#ifndef _LOG_H_
#define _LOG_H_

#pragma pack(push,4)

struct ASM_APIFunctionStub;
struct ReturnStub;

/*
struct OneParamInfo is used to hold information about one
parameter of a function during the process of formatting
of function parameters
*/
struct FunctionParameters;
enum ParamType;
struct OneParamInfo
{
    OneParamInfo
    (
        ParamType    p_ParamType,
        UCHAR        p_byParamFlags,
        UCHAR        p_byExtraByte,
        ULONG    *    p_pdwAddrCurParameter//may be NULL 
            //if we processing parameters on function's return
    );
    void vSetValue(ULONG p_dwValue)
    {
        m_dwCurParamValue = p_dwValue;
        m_pdwAddrCurParameter = &m_dwCurParamValue;
    }

    int iGetParamSize(void){return m_iParamSize;}

    ParamType    m_ParamType;
    UCHAR        m_byParamFlags;
    UCHAR        m_byExtraByte;
    ULONG    *    m_pdwAddrCurParameter;
    ULONG       m_dwCurParamValue;
    
    int         m_iParamSize;//Return value - number of bytes
                         //occupied by this parameter in the
                         //memory pointed by m_pdwAddrCurParameter.
                         //Useful for pointer adjustment
};//struct OneParamInfo


/*
struct ParamProcContext holds information about function
parameters during the process of processing them.
*/
struct ParamProcContext
{
    void vInit(
        const FunctionParameters * p_pParams,
        ReturnStub               * p_pReturnStub,
        ULONG    * p_pdwCallStackFrams,//may be NULL
        bool                       p_bOut    
        );

    int iPrintParameter
    (
        OneParamInfo & p_rParam,
        PSTR    p_pszReturnPrintedParams,
        int        p_iParamsBuffSize,
        int        p_iPrintIntegerBytes,
        PCSTR    p_pszParamType
    );

    int iProcParameter
    (
        OneParamInfo & p_rParam,
        bool            p_bParamFromStack,
        PSTR  p_pszReturnPrintedParams,//must be a valid pointer
        int            p_iParamsBuffSize
    );
    int iProcAllParams
    (
        PSTR    p_pszReturnPrintedParams,//must be a valid pointer
        int        p_iParamsBuffSize
    );
    bool bStoreRetreiveOutParameter
    (
        bool         p_bStore,
        ULONG      & p_rdwParameter,
        ReturnStub * p_pReturnStub,
        int        * p_piOutputParamIndex
    );

    int iGetLPSTR
    ( 
       PSTR  p_pStr,
       int   p_iCharSize,
       int   p_iNumCharsToPrint,//if -1, print up to '\0'
       PSTR  p_pszOutputBuffer,
       int   p_iSizeOutBuffer
    );
    int iGetDDKString
    (
        UNICODE_STRING * p_pUStr,
        PSTR p_pszOutputBuffer,
        int  p_iSizeOutBuffer,
        int  p_iCharacterSize
    );

    const FunctionParameters * m_pParams;
    ReturnStub               * m_pReturnStub;
    PULONG        m_pCallStackFrame;//may be NULL
    bool        m_bOut;

    ULONG        m_dwThisAddress;//this is the only "return" value

    int            m_iOutputParamIndex;
            //every time we process an [OUT]
            //parameter we must store the pointer
            //in our own array inside m_rReturnStub.
            //m_iOutputParamIndex - is the next 
            //index in this array.

};//struct ParamProcContext

class Logger
{
public:
    static void vLogFunction
    (
        const char       * p_pszFunctionName,
        ParamProcContext * p_pParamContext,
        bool               p_bNotEnoughStubs,
        char       *       p_pszBuffer,
        int                p_iSizeBuffer
    );
    static void LogReturn
    (
        const char       * p_pszFunctionName,
        ParamProcContext * p_pParamContext,
        bool               p_bOutputParamsExist,
        ULONG              p_dwReturnValue,
        char       *       p_pszBuffer,
        int                p_iSizeBuffer
    );
    static void ProcessReturn
    (
        ASM_APIFunctionStub * p_pStub,
        ULONG                  p_dwReturnValue, 
        ReturnStub          *  p_pReturnStub
    );
    static void __stdcall IntrerceptEntry
    (
        ASM_APIFunctionStub * p_pStub,
        PULONG  p_pdwFrame
    );
    static void IntrerceptEntryNaked();
    static int iMakePrefixString
    (
        PSTR  p_pszBuffer,
        int   p_iBuffSize
    );
    static void PrintStatistics();
};//class Logger

#pragma pack(pop)

#endif//_LOG_H_
