/******************************************************************
Module : ParmType.cpp      Support for encoded function parameters
             Written 1998,1999 by Dmitri Leman
Purpose: COM/OLE tracer and NT kernel mode tracer.
******************************************************************/
#include "Headers.h"
#include "Driver.h"
#include "KrnlTrcD.h"
#include "parmtype.h"
#include "Trace.h"

/*----------------------------------------------------------------

   FUNCTION: FunctionParameters::GetParam

   PURPOSE:  Get one parameter
      
   PARAMETERS:                   .
                                  
   int p_iArrayIndex             Index of the parameter in the
                                 array of encoded bytes
   int & p_riReturnNextIndex     return index of the next parameter
                                 in the array

   RETURN VALUE:
      ParamType                  Type of the parameter
-----------------------------------------------------------------*/
ParamType FunctionParameters::GetParam
(
    int p_iArrayIndex, 
    int & p_riReturnNextIndex
) const
{
    if(ULONG(p_iArrayIndex) >= ULONG(m_byNumBytes))
    {
        p_riReturnNextIndex = -1;
        return PARAM_NONE;
    }
    UCHAR l_byParam = m_byParams[p_iArrayIndex];
    l_byParam &= ~PF_AllFlags;

    p_riReturnNextIndex = p_iArrayIndex+1;

    if(bIs2ByteParam(ParamType(l_byParam)))
    {
        //This is a special parameters which
        //occupies 2 bytes
        p_riReturnNextIndex++;
    }
    return ParamType(l_byParam);
}//ParamType FunctionParameters::GetParam

/*-----------------------------------------------------------------

   FUNCTION: FunctionParameters::byGetParamFlags

   PURPOSE:  Get flags for the parameter
      
   PARAMETERS:                   .
                                  
   int p_iArrayIndex              Index of the parameter in the
                                 array of encoded bytes

   RETURN VALUE:
      UCHAR           Flags
-----------------------------------------------------------------*/

UCHAR FunctionParameters::byGetParamFlags
(
    int p_iArrayIndex
) const
{
    if(ULONG(p_iArrayIndex) >= ULONG(m_byNumBytes))
        return 0;
    return (m_byParams[p_iArrayIndex] & PF_AllFlags);
}//UCHAR FunctionParameters::byGetParamFlags

/*-----------------------------------------------------------------

   FUNCTION: FunctionParameters::byGetParamExtraByte

   PURPOSE:  If the parameter has an extra byte, get it.
    Use bIs2ByteParam to determine.
      
   PARAMETERS:                   .
                                  
   int p_iArrayIndex              Index of the parameter in the
                                 array of encoded bytes

   RETURN VALUE:
      UCHAR               return the byte
-----------------------------------------------------------------*/
UCHAR FunctionParameters::byGetParamExtraByte
(
    int p_iArrayIndex
) const
{
    if(ULONG(p_iArrayIndex)+1 >= ULONG(m_byNumBytes))
        return 0;
    if(!bIs2ByteParam(ParamType(
        m_byParams[p_iArrayIndex] & ~PF_AllFlags)))
        return 0;
    return m_byParams[p_iArrayIndex+1];
}//UCHAR FunctionParameters::byGetParamSecondByte




