/******************************************************************
Module : ParmType.h     Support for encoded function parameters
             Written 1998,1999 by Dmitri Leman
Purpose: COM/OLE tracer, NT kernel mode tracer.
******************************************************************/

#ifndef _PARMTYPE_H_
#define _PARMTYPE_H_

#include "KrnlTrc.h"

#pragma pack(push,1)

enum ParamType
{
PARAM_NONE      = 0,
PARAM_DWORD     = 1,
PARAM_WORD      = 2,
PARAM_BYTE      = 3,
PARAM_FLOAT     = 4,
PARAM_DOUBLE    = 5,
PARAM_DWORD64   = 6,
PARAM_LPSTR     = 7,
PARAM_LPWSTR    = 8,
PARAM_LPDATA    = 9,
PARAM_HANDLE    = 10,
PARAM_HWND      = 11,
PARAM_BOOL      = 12,
PARAM_LPCODE    = 13,
PARAM_LPIUNKNOWN= 14,
PARAM_LPINTERFACE=15,
PARAM_LPGUID    = 16,
PARAM_BSTR      = 17,
PARAM_THIS      = 18,
PARAM_VARIANT   = 19,
PARAM_U_STR     = 20, //UNICODE_STRING structure from ntdef.h
PARAM_A_STR     = 21, //ANSI_STRING structure from ntdef.h
//must be no more than 32 types because flags
//PF_Out, PF_In and PF_Pointer will be applied.
//When I will need more than 32 types, I will rewrite the program 
//to use WORD instead of UCHAR to encode parameter type.
//I don't expect it to happen soon (Y2K event will come first).
};//enum ParamType

inline bool bIs2ByteParam(ParamType p_Type)
{return (p_Type == PARAM_LPIUNKNOWN ||
         p_Type == PARAM_LPINTERFACE)? TRUE : FALSE;}

enum ParamFlags
{
    PF_Pointer = 0x20,
    PF_Out = 0x40,
    PF_In  = 0x80,
    PF_AllFlags = (PF_Pointer | PF_Out | PF_In)
};//enum ParamFlags

#define MAX_NUM_BYTES_PARAMS 63
#define MAX_NUM_PARAMS       31

class ParamsUnderConstr;
struct FunctionParameters
{
    FunctionParameters(void) {vClear();}
    void vClear(void){m_byNumParameters = m_byNumBytes = 0;}

    int iGetTotalFilledBytes(void) const
        {return m_byNumBytes + 
        offsetof(FunctionParameters, m_byParams);}

    UCHAR byGetNumParams(void) const
        {return (m_byNumParameters & ~PF_AllFlags);}
    UCHAR byGetFlags(void) const
        {return (m_byNumParameters & PF_AllFlags);}

    ParamType GetParam(int p_iArrayIndex, 
        int & p_riReturnNextIndex) const;
    UCHAR byGetParamFlags(int p_iArrayIndex) const;
    UCHAR byGetParamExtraByte(int p_iArrayIndex) const;
protected:
    UCHAR    m_byNumParameters;//low 5 bits holds the number,
                         //high 3 bits holds flags from ParamFlags
    UCHAR    m_byNumBytes;//total number of UCHARs used by m_byParams
    UCHAR    m_byParams[1];
                          //Not all UCHARs may be actually
                          //allocated. 
                          //Actual number of UCHARs in this
                          //array may be equal or more
                          //then the number of parameters
                          //because some parameters are
                          //encoded by 2 UCHARs
    friend ParamsUnderConstr;
};//struct FunctionParameters

#pragma pack(pop)

#endif//_PARMTYPE_H_
