/******************************************************************
Module : Trace.h    Support for debug trace and output file logging
             Written 1998,1999 by Dmitri Leman
Purpose: COM/OLE tracer, NT kernel mode tracer
******************************************************************/

#ifndef _TRACE_H_
#define _TRACE_H_

enum TraceFlags
{
    TF_MainLog    = 1,//Trace of intercepted COM and API calls
                      //Normally redirected to a file
    
//The following flags are for debugging and diagnostics:

    TF_Driver      = 0x1000, //NT driver routines
    TF_SpyInit     = 0x2000, //Initialization of Spy DLL(or driver)
    TF_ApiHooked   = 0x4000, //Functions hooked in a module
    TF_ListAllModules = 0x8000, //module enumeration
    TF_Intercepted    = 0x20000, //interception of a function

//The following are flags for error messages:
    TF_MessageBox =  0x1000000,//errors and warnings in KrnlTrc.EXE

    TF_Error      = 0x80000000 //errors
};//enum TraceFlags

enum TraceTypes
{
    TT_DebugMonitor = 1,
    TT_File         = 2,
    TT_FlushFile    = 4,
    TT_MessageBox   = 8,
    TT_MemoryBuffer = 0x10,
    TT_OutputLog    = 0x20
};//enum TraceTypes

#ifdef _TRACE
void vTraceInitialize(void);
void vTraceUnInitialize(void);

void vSetCurTraceMasks
(
    ULONG   p_dwTraceTypes,
    ULONG   p_dwNewBits   ,
    ULONG   p_dwBitsToModify

);

#ifndef _DRIVER
void vSetOutputToFile
(
    LPCTSTR p_pszFileName,
    ULONG   p_dwNewBits   ,
    ULONG   p_dwBitsToModify,
    ULONG   p_dwNewFlushBits   ,
    ULONG   p_dwFlushBitsToModify
);
#endif

bool bAllocateTraceBuffer(int p_iSize,
    ULONG   p_dwNewBits, ULONG   p_dwBitsToModify);
bool bFreeTraceBuffer();
bool bDumpTraceBufferToFile(const char * p_pszFileName);
#ifdef _DRIVER
typedef struct _DRIVER_OBJECT *PDRIVER_OBJECT;
bool bEnableTraceToOutputLog(PDRIVER_OBJECT p_pDriverObject,
    ULONG   p_dwNewBits, ULONG   p_dwBitsToModify);
#endif

void MyTraceImpl(ULONG p_dwMask, LPCTSTR p_pszFormat, ...);
void OutputTraceString(ULONG p_dwMask, LPCTSTR p_pszMessage, 
                       int p_iLen);

#define MYTRACE MyTraceImpl

#else //_TRACE
inline void vTraceInitialize(){};
inline void vTraceUnInitialize(){};

inline void vSetCurTraceMasks(ULONG   p_dwTraceTypes,
    ULONG   p_dwNewBits, ULONG   p_dwBitsToModify) {};
inline void OutputTraceString(ULONG p_dwMask, LPCTSTR p_pszMessage,
                              int     p_iLen){}
inline void MyTraceImpl(ULONG p_dwMask, LPCTSTR p_pszFormat,...){}
#define MYTRACE            1 ? (void)0 : ::MyTraceImpl

#endif//_TRACE

#ifndef _DRIVER //{
class LocalStringHolder
{
public:
    LocalStringHolder()
    {
        m_pszString = m_szDummy;
        *m_szDummy = 0;
    }
    ~LocalStringHolder(){vFree();}
    LocalStringHolder(const LocalStringHolder & p_rScr) 
    {
        m_pszString = m_szDummy;
        *m_szDummy = 0;
        vStrDup(p_rScr.m_pszString);
    }
    LocalStringHolder & operator = 
        (const LocalStringHolder & p_rScr) 
    {
        vFree();
        vStrDup(p_rScr.m_pszString);
        return *this;
    }
    void vStrDup(PTSTR p_pszStr)
    {
        vFree();
        int l_iSize = (_tcslen(p_pszStr)+1)*sizeof(TCHAR);
        m_pszString = (PTSTR)LocalAlloc(LPTR, l_iSize);
        if(!m_pszString)
            m_pszString = m_szDummy;
        else
            memcpy(m_pszString, p_pszStr, l_iSize);
    }//void vSetStr

    void vAdoptStr(PTSTR p_pszStr)
        //p_pszStr must be allocated using LocalAlloc
    {
        vFree();
        m_pszString = p_pszStr;
    }

    LPCTSTR strGetStr(){return m_pszString;}

    void vFree()
    {
        if(m_pszString != m_szDummy)
            LocalFree(m_pszString);
        m_pszString = m_szDummy;
    }//void vFree()
protected:
    PTSTR   m_pszString;
    TCHAR   m_szDummy[1];
};//class LocalStringHolder

LocalStringHolder strGetErrorExplanation(DWORD p_dwErrCode);
#define ERR_EXPL(ErrorCode) \
    ((LPCTSTR)strGetErrorExplanation(ErrorCode).strGetStr())
#endif //_DRIVER }

#endif //_TRACE_H_
